// $Id: main.c 333 2008-05-10 20:49:56Z tk $
/*
 * MIOS SDCC Wrapper
 *
 * ==========================================================================
 *
 *  Copyright (C) <year>  <name> (<email>)
 *  Licensed for personal non-commercial use only.
 *  All other rights reserved.
 * 
 * ==========================================================================
 */


/////////////////////////////////////////////////////////////////////////////
// Include files
/////////////////////////////////////////////////////////////////////////////

#include <cmios.h>
#include <pic18fregs.h>
#include <iic_dmx.h>
#include "main.h"
#include "mdmx_lcd.h"
#include "mdmx_midi.h"
#include "mdmx_dmx.h"


/////////////////////////////////////////////////////////////////////////////
// Global variables
/////////////////////////////////////////////////////////////////////////////

// status of MM emulation
mdmx_flags_t mdmx_flags;


/////////////////////////////////////////////////////////////////////////////
// This function is called by MIOS after startup to initialize the 
// application
/////////////////////////////////////////////////////////////////////////////
void Init(void) __wparam
{
	unsigned char f;
	MIOS_LCD_YAddressSet(0x00,0x40,0x80,0xc0);
	MIOS_AIN_Muxed();
	MIOS_AIN_NumberSet(0x08);
	MIOS_AIN_DeadbandSet(0x03);
	IIC_DMX_Init(); // Initialize IIC Module

	MIOS_IIC_CtrlSet(0x01); // enable clock stretching
	//if (!MDMX_DMX_Init(MDMX_DMX_INTERFACE))
	//	MIOS_LCD_PrintMessage("** NO DMX I/F **");
   	if(!iic_dmx_available_slaves)   
		MIOS_LCD_PrintCString("**NO IIC Found**");

	// request display initialization
	mdmx_flags.DISPLAY_INIT_REQ = 1;

	
}

/////////////////////////////////////////////////////////////////////////////
// This function is called by MIOS in the mainloop when nothing else is to do
/////////////////////////////////////////////////////////////////////////////
void Tick(void) __wparam
{
	unsigned char byte,f;
	

 	if (!IIC_DMX_Receive(0x10))
		return;
		
	MIOS_MIDI_BeginStream();
	MIOS_MIDI_TxBufferPut(iic_dmx_rx_package[0]);
	if (iic_dmx_rx_package[1]<128)
		MIOS_MIDI_TxBufferPut(iic_dmx_rx_package[1]);
	else
		MIOS_MIDI_TxBufferPut(iic_dmx_rx_package[1]-127);
	MIOS_MIDI_TxBufferPut(iic_dmx_rx_package[2] >> 1);
	//MIOS_MIDI_TxBufferPut(0x0);
	MIOS_MIDI_EndStream();

}

/////////////////////////////////////////////////////////////////////////////
// This function is periodically called by MIOS. The frequency has to be
// initialized with MIOS_Timer_Set
/////////////////////////////////////////////////////////////////////////////
void Timer(void) __wparam
{
}


/////////////////////////////////////////////////////////////////////////////
// This function is called by MIOS when the display content should be 
// initialized. Thats the case during startup and after a temporary message
// has been printed on the screen
/////////////////////////////////////////////////////////////////////////////
void DISPLAY_Init(void) __wparam
{
  // clear screen
  MIOS_LCD_Clear();
  mdmx_flags.REFRESH_MENU=0;
  // request display update
  mdmx_flags.DISPLAY_UPDATE_REQ = 1;
}

/////////////////////////////////////////////////////////////////////////////
//  This function is called in the mainloop when no temporary message is shown
//  on screen. Print the realtime messages here
/////////////////////////////////////////////////////////////////////////////
void DISPLAY_Tick(void) __wparam
{
  // check if display initialization has been requested
  if( mdmx_flags.DISPLAY_INIT_REQ ) {
    mdmx_flags.DISPLAY_INIT_REQ = 0;
    
    MDMX_LCD_Init();
  }

  // update screen if requested

  if( mdmx_flags.DISPLAY_UPDATE_REQ  && !mdmx_flags.REFRESH_MENU) 
  {
    mdmx_flags.DISPLAY_UPDATE_REQ = 0;
    MDMX_LCD_Msg_Refresh();
  }
}

/////////////////////////////////////////////////////////////////////////////
//  This function is called by MIOS when a complete MIDI event has been received
/////////////////////////////////////////////////////////////////////////////
void MPROC_NotifyReceivedEvnt(unsigned char evnt0, unsigned char evnt1, unsigned char evnt2) __wparam
{
  // forward MIDI event to MIDI handler
  MDMX_MIDI_Received(evnt0, evnt1, evnt2);
}

/////////////////////////////////////////////////////////////////////////////
// This function is called by MIOS when a MIDI event has been received
// which has been specified in the MIOS_MPROC_EVENT_TABLE
/////////////////////////////////////////////////////////////////////////////
void MPROC_NotifyFoundEvent(unsigned entry, unsigned char evnt0, unsigned char evnt1, unsigned char evnt2) __wparam
{
}

/////////////////////////////////////////////////////////////////////////////
// This function is called by MIOS when a MIDI event has not been completly
// received within 2 seconds
/////////////////////////////////////////////////////////////////////////////
void MPROC_NotifyTimeout(void) __wparam
{
}

/////////////////////////////////////////////////////////////////////////////
// This function is called by MIOS when a MIDI byte has been received
/////////////////////////////////////////////////////////////////////////////
void MPROC_NotifyReceivedByte(unsigned char byte) __wparam
{
}

/////////////////////////////////////////////////////////////////////////////
// This function is called by MIOS before the shift register are loaded
/////////////////////////////////////////////////////////////////////////////
void SR_Service_Prepare(void) __wparam
{
}

/////////////////////////////////////////////////////////////////////////////
// This function is called by MIOS after the shift register have been loaded
/////////////////////////////////////////////////////////////////////////////
void SR_Service_Finish(void) __wparam
{
}

/////////////////////////////////////////////////////////////////////////////
// This function is called by MIOS when an button has been toggled
// pin_value is 1 when button released, and 0 when button pressed
/////////////////////////////////////////////////////////////////////////////
void DIN_NotifyToggle(unsigned char pin, unsigned char pin_value) __wparam
{
}

/////////////////////////////////////////////////////////////////////////////
// This function is called by MIOS when an encoder has been moved
// incrementer is positive when encoder has been turned clockwise, else
// it is negative
/////////////////////////////////////////////////////////////////////////////
void ENC_NotifyChange(unsigned char encoder, char incrementer) __wparam
{
}

/////////////////////////////////////////////////////////////////////////////
// This function is called by MIOS when a pot has been moved
/////////////////////////////////////////////////////////////////////////////
void AIN_NotifyChange(unsigned char pin, unsigned int pin_value) __wparam
{
	if (pin==0x07)
		MDMX_DMX_Update_Master(pin_value >> 2);
	else
		MDMX_DMX_Update_Channel(0x0, pin, pin_value >> 2);
}
